@@ -48,8 +48,7 @@ data TypeInfo =
   -- struct.unpack string, second is the size.
   BaseType String |
   -- | A composite type, i.e. a Struct or Union created by XCB. First arg is
-  -- the extension that defined it, second is the name of the type, third arg
-  -- is the size if it is known.
+  -- the extension that defined it, second is the name of the type.
   CompositeType String String
   deriving (Eq, Ord, Show)
 
@@ -326,7 +325,7 @@ structElemToPyPack :: String
                    -> TypeInfoMap
                    -> (String -> String)
                    -> GenStructElem Type
-                   -> Either (Maybe String, String) [(String, Expr ())]
+                   -> Either (Maybe String, String) [(String, Maybe (Expr ()))]
 structElemToPyPack _ _ _ (Pad i) = Left (Nothing, mkPad i)
 -- TODO: implement doc, switch, and fd?
 structElemToPyPack _ _ _ (Doc _ _ _) = Left (Nothing, "")
@@ -336,34 +335,39 @@ structElemToPyPack _ m accessor (SField n typ _ _) =
   let name = accessor n
   in case m M.! typ of
        BaseType c -> Left (Just name, c)
-       -- XXX: be a little smarter here? we should really make sure that things
-       -- have a .pack(); if users are calling us via the old style api, we need
-       -- to support that as well. This isn't super necessary, though, because
-       -- currently (xcb-proto 1.10) there are no direct packs of raw structs, so
-       -- this is really only necessary if xpyb gets forward ported in the future if
-       -- there are actually calls of this type.
-       CompositeType _ _ -> Right $ [(name, mkCall (name ++ ".pack") noArgs)]
+       CompositeType _ typNam ->
+         let cond = mkCall "hasattr" [mkArg name, ArgExpr (mkStr "pack") ()]
+             trueB = mkCall (name ++ ".pack") noArgs
+             synthetic = mkCall (typNam ++ ".synthetic") [mkArg ("*" ++ name)]
+             falseB = mkCall (mkDot synthetic "pack") noArgs
+         in Right $ [(name, Just (CondExpr trueB cond falseB ()))]
 -- TODO: assert values are in enum?
-structElemToPyPack ext m accessor (X.List n typ _ _) =
+structElemToPyPack ext m accessor (X.List n typ expr _) =
   let name = accessor n
-  in case m M.! typ of
-        BaseType c -> Right $ [(name, mkCall "xcffib.pack_list" [ mkName $ name
-                                                                , mkStr c
-                                                                ])]
+      -- The convention seems to be either to have a <fieldref> nested in the
+      -- list, or use "%s_len" % name if there is no fieldref. We need to add
+      -- the _len to the arguments of the function but we don't need to pack
+      -- anything, which we denote using Nothing
+      list_len = if isNothing expr then [(name ++ "_len", Nothing)] else []
+      list = case m M.! typ of
+        BaseType c -> [(name, Just (mkCall "xcffib.pack_list" [ mkName $ name
+                                                        , mkStr c
+                                                        ]))]
         CompositeType tExt c ->
           let c' = if tExt == ext then c else (tExt ++ "." ++ c)
-          in Right $ [(name, mkCall "xcffib.pack_list" ([ mkName $ name
-                                                        , mkName c'
-                                                        ]))]
+          in [(name, Just (mkCall "xcffib.pack_list" ([ mkName $ name
+                                                , mkName c'
+                                                ])))]
+  in Right $ list_len ++ list
 structElemToPyPack _ m accessor (ExprField name typ expr) =
   let e = (xExpressionToPyExpr accessor) expr
       name' = accessor name
   in case m M.! typ of
-       BaseType c -> Right $ [(name', mkCall "struct.pack" [ mkStr ('=' : c)
+       BaseType c -> Right $ [(name', Just (mkCall "struct.pack" [ mkStr ('=' : c)
                                                            , e
-                                                           ])]
+                                                           ]))]
        CompositeType _ _ -> Right $ [(name',
-                                      mkCall (mkDot e "pack") noArgs)]
+                                      Just (mkCall (mkDot e "pack") noArgs))]
 
 -- As near as I can tell here the padding param is unused.
 structElemToPyPack _ m accessor (ValueParam typ mask _ list) =
@@ -373,7 +377,7 @@ structElemToPyPack _ m accessor (ValueParam typ mask _ list) =
           list' = mkCall "xcffib.pack_list" [ mkName $ accessor list
                                             , mkStr "I"
                                             ]
-      in Right $ [(mask, mask'), (list, list')]
+      in Right $ [(mask, Just mask'), (list, Just list')]
     CompositeType _ _ -> error (
       "ValueParams other than CARD{16,32} not allowed.")
 
@@ -390,7 +394,7 @@ mkPackStmts :: String
 mkPackStmts ext name m accessor prefix membs =
   let packF = structElemToPyPack ext m accessor
       (toPack, stmts) = partitionEithers $ map packF membs
-      listWrites = map (flip StmtExpr () . mkCall "buf.write" . (: [])) lists
+      listWrites = map (flip StmtExpr () . mkCall "buf.write" . (: [])) $ catMaybes lists
       (args, keys) = let (as, ks) = unzip toPack in (catMaybes as, ks)
 
       -- In some cases (e.g. xproto.ConfigureWindow) there is padding after
@@ -401,12 +405,12 @@ mkPackStmts ext name m accessor prefix membs =
       -- implicitly.
       (listNames, lists) = unzip $ filter (flip notElem args . fst) (concat stmts)
       listNames' = case (ext, name) of
-                     -- XXX: QueryTextExtents has a field named "odd_length" with a
-                     -- fieldref of "string_len", so we fix it up here to match.
+                     -- XXX: QueryTextExtents has a field named "odd_length"
+                     -- which is unused, let's just drop it.
                      ("xproto", "QueryTextExtents") ->
-                       let replacer "odd_length" = "string_len"
-                           replacer s = s
-                       in map replacer listNames
+                       let notOdd "odd_length" = False
+                           notOdd _ = True
+                       in filter notOdd listNames
                      _ -> listNames
       packStr = addStructData prefix $ intercalate "" keys
       write = mkCall "buf.write" [mkCall "struct.pack"
@@ -535,6 +539,30 @@ mkModify ext name ti m =
                       ]
   in M.union m m'
 
+mkSyntheticMethod :: [GenStructElem Type] -> [Statement ()]
+mkSyntheticMethod membs = do
+  let names = catMaybes $ map getName membs
+      args = mkParams $ "cls" : names
+      self = mkAssign "self" $ mkCall (mkDot "cls" "__new__") [mkName "cls"]
+      body = map assign names
+      ret = mkReturn $ mkName "self"
+      synthetic = mkMethod "synthetic" args $ (self : body) ++ [ret]
+      classmethod = Decorator [ident "classmethod"] noArgs ()
+  if null names then [] else [Decorated [classmethod] synthetic ()]
+    where
+      getName :: GenStructElem Type -> Maybe String
+      getName (Pad _) = Nothing
+      getName (X.List n _ _ _) = Just n
+      getName (SField n _ _ _) = Just n
+      getName (ExprField n _ _) = Just n
+      getName (ValueParam _ n _ _) = Just n
+      getName (Switch n _ _) = Just n
+      getName (Doc _ _ _) = Nothing
+      getName (Fd n) = Just n
+
+      assign :: String -> Statement ()
+      assign n = mkAssign (mkDot "self" n) $ mkName n
+
 processXDecl :: String
              -> XDecl
              -> State TypeInfoMap BindingPart
@@ -553,20 +581,22 @@ processXDecl ext (XStruct n membs) = do
   m <- get
   let (statements, len) = mkStructStyleUnpack "" ext m membs
       pack = mkPackMethod ext n m Nothing membs Nothing
+      synthetic = mkSyntheticMethod membs
       fixedLength = maybeToList $ do
         theLen <- len
         let rhs = mkInt theLen
         return $ mkAssign "fixed_size" rhs
   modify $ mkModify ext n (CompositeType ext n)
-  return $ Declaration [mkXClass n "xcffib.Struct" statements (pack : fixedLength)]
+  return $ Declaration [mkXClass n "xcffib.Struct" statements (pack : fixedLength ++ synthetic)]
 processXDecl ext (XEvent name opcode membs noSequence) = do
   m <- get
   let cname = name ++ "Event"
       prefix = if fromMaybe False noSequence then "x" else "x%c2x"
       pack = mkPackMethod ext name m (Just (prefix, opcode)) membs (Just 32)
+      synthetic = mkSyntheticMethod membs
       (statements, _) = mkStructStyleUnpack prefix ext m membs
       eventsUpd = mkDictUpdate "_events" opcode cname
-  return $ Declaration [ mkXClass cname "xcffib.Event" statements [pack]
+  return $ Declaration [ mkXClass cname "xcffib.Event" statements (pack : synthetic)
                        , eventsUpd
                        ]
 processXDecl ext (XError name opcode membs) = do
@@ -586,7 +616,7 @@ processXDecl ext (XRequest name opcode membs reply) = do
   let
       -- xtest doesn't seem to use the same packing strategy as everyone else,
       -- but there is no clear indication in the XML as to why that is. yay.
-      prefix = if ext == "xtest" then "xx2x" else "x%c2x"
+      prefix = if ext /= "xproto" then "xx2x" else "x%c2x"
       (args, packStmts) = mkPackStmts ext name m id prefix membs
       cookieName = (name ++ "Cookie")
       replyDecl = concat $ maybeToList $ do
@@ -605,12 +635,12 @@ processXDecl ext (XRequest name opcode membs reply) = do
       argChecked = ArgKeyword (ident "is_checked") (mkName "is_checked") ()
       checkedParam = Param (ident "is_checked") Nothing (Just isChecked) ()
       allArgs = (mkParams $ "self" : args) ++ [checkedParam]
-      mkArg = flip ArgExpr ()
-      ret = mkReturn $ mkCall "self.send_request" ((map mkArg [ mkInt opcode
-                                                              , mkName "buf"
-                                                              ])
-                                                              ++ hasReply
-                                                              ++ [argChecked])
+      mkArg' = flip ArgExpr ()
+      ret = mkReturn $ mkCall "self.send_request" ((map mkArg' [ mkInt opcode
+                                                               , mkName "buf"
+                                                               ])
+                                                               ++ hasReply
+                                                               ++ [argChecked])
       requestBody = buf ++ packStmts ++ [ret]
       request = mkMethod name allArgs requestBody
   return $ Request request replyDecl
