@@ -16,6 +16,7 @@
 from __future__ import division, absolute_import
 
 import functools
+import platform
 import six
 import struct
 import weakref
@@ -25,7 +26,15 @@ try:
 except ImportError:
     from xcffib.ffi_build import ffi
 
-lib = ffi.dlopen('libxcb.so.1')
+SONAMES = {
+    "Linux": "libxcb.so.1",
+    "Darwin": "libxcb.dylib",
+}
+
+try:
+    lib = ffi.dlopen(SONAMES[platform.system()])
+except KeyError:
+    raise NotImplementedError("unsupported platform %s" % platform.system())
 
 __xcb_proto_version__ = 'placeholder'
 
@@ -84,8 +93,7 @@ class Unpacker(object):
             self._resize(known_max)
 
     def pad(self, thing):
-        if isinstance(thing, type) and any(
-                [issubclass(thing, c) for c in [Struct, Union]]):
+        if isinstance(thing, type) and issubclass(thing, (Struct, Union)):
             if hasattr(thing, "fixed_size"):
                 size = thing.fixed_size
             else:
@@ -290,13 +298,6 @@ class Protobj(object):
         if unpacker.known_max is not None:
             self.bufsize = unpacker.known_max
 
-    @classmethod
-    def synthetic(cls, **kwargs):
-        self = cls.__new__(cls)
-        for k, v in kwargs.items():
-            setattr(self, k, v)
-        return self
-
 
 class Struct(Protobj):
     pass
@@ -452,6 +453,15 @@ class List(Protobj):
     def buf(self):
         return b''.join(self.list)
 
+    @classmethod
+    def synthetic(cls, list=None):
+        if list is None:
+            list = []
+
+        self = cls.__new__(cls)
+        self.list = list[:]
+        return self
+
 
 class OffsetMap(object):
 
@@ -753,8 +763,10 @@ def pack_list(from_, pack_type):
     else:
         buf = six.BytesIO()
         for item in from_:
-            # If we can't pack it, you'd better have packed it yourself...
-            if isinstance(item, Struct):
+            # If we can't pack it, you'd better have packed it yourself. But
+            # let's not confuse things which aren't our Probobjs for packable
+            # things.
+            if isinstance(item, Protobj) and hasattr(item, "pack"):
                 buf.write(item.pack())
             else:
                 buf.write(item)
